/*
 *  libpipi       Proper image processing implementation library
 *  Copyright (c) 2004-2008 Sam Hocevar <sam@zoy.org>
 *                All Rights Reserved
 *
 *  $Id$
 *
 *  This library is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

/*
 * screen.c: halftoning screen functions
 */

#include "config.h"
#include "common.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "pipi.h"
#include "pipi_internals.h"

pipi_image_t *pipi_render_bayer(int w, int h)
{
    pipi_image_t *ret;
    pipi_pixels_t *pix;
    float *data;
    int i, j, n;

    if(w <= 0 || h <= 0)
        return NULL;

    for(n = 1; n < w || n < h; n *= 2)
        ;

    ret = pipi_new(w, h);
    pix = pipi_getpixels(ret, PIPI_PIXELS_Y_F);
    data = (float *)pix->pixels;

    for(j = 0; j < h; j++)
        for(i = 0; i < w; i++)
        {
            int k, l, x = 0;

            for(k = 1, l = n * n / 4; k < n; k *= 2, l /= 4)
            {
                if((i & k) && (j & k))
                    x += l;
                else if(i & k)
                    x += 3 * l;
                else if(j & k)
                    x += 2 * l;
            }

            data[j * w + i] = (double)(x + 1) / (n * n + 1);
        }

    return ret;
}

